﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Web;
using System.Web.UI;
using System.Web.UI.HtmlControls;
using System.Web.UI.WebControls;

namespace TSC.Timeout
{
    [ToolboxData("<{0}:Timeout runat=server></{0}:Timeout>")]
    public class Timeout : WebControl, INamingContainer, IScriptControl, ICallbackEventHandler
    {
        #region Properties

        [Category("Behavior")]
        [DefaultValue(15)]
        [Description("Idle minutes until the membership session will time out")]
        public float TimeoutMinutes
        {
            get
            {
                object o = ViewState["TimeoutMinutes"];
                if (o != null)
                    return (float) o;
                else
                    return 15;
            }

            set { ViewState["TimeoutMinutes"] = value; }
        }

        [Category("Behavior")]
        [DefaultValue(14)]
        [Description("Idle minutes until the user is informed that their session is about to time out")]
        public float AboutToTimeoutMinutes
        {
            get
            {
                object o = ViewState["AboutToTimeoutMinutes"];
                if (o != null)
                    return (float) o;
                else
                    return 14;
            }

            set { ViewState["AboutToTimeoutMinutes"] = value; }
        }


        [Category("Behavior")]
        [DefaultValue("")]
        [UrlProperty]
        [Description("URL to redirect the user, in the event of a membership session timeout")]
        public string TimeoutURL
        {
            get
            {
                object o = ViewState["TimeoutURL"];
                if (o != null)
                    return (string) o;
                else
                    return String.Empty;
            }

            set { ViewState["TimeoutURL"] = value; }
        }

        [Category("Appearance")]
        [DefaultValue("Continue Session")]
        [Description("The text to be displayed on the button")]
        public string ButtonText
        {
            get
            {
                object o = ViewState["ButtonText"];
                if (o != null)
                    return (string) o;
                else
                    return "Continue Session";
            }

            set { ViewState["ButtonText"] = value; }
        }

        [Category("Appearance")]
        [DefaultValue("")]
        [Description("Css class name applied to the button control")]
        public string ButtonCssClass
        {
            get
            {
                object o = ViewState["ButtonCssClass"];
                if (o != null)
                    return (string) o;
                else
                    return String.Empty;
            }

            set { ViewState["ButtonCssClass"] = value; }
        }

        [Category("Appearance")]
        [DefaultValue("True")]
        [Description("Whether the button is visible.")]
        public bool DisplayButton
        {
            get
            {
                object o = ViewState["DisplayButton"];
                if (o != null)
                    return (bool) o;
                else
                    return true;
            }

            set { ViewState["DisplayButton"] = value; }
        }

        #endregion

        private ScriptManager _sm;
        private ITemplate pSimpleTemplate;
        private HtmlInputButton btn;

        [Browsable(false)]
        [PersistenceMode(PersistenceMode.InnerProperty)]
        public ITemplate Template
        {
            get { return pSimpleTemplate; }
            set { pSimpleTemplate = value; }
        }

        protected override void CreateChildControls()
        {
            if (this.Enabled)
            {
                Controls.Clear();
                if (Template != null) Template.InstantiateIn(this);
                if (DisplayButton)
                {
                    btn = new HtmlInputButton();
                    btn.ID = "btnContinue";
                    btn.Value = ButtonText;
                    btn.Attributes.Add("class", ButtonCssClass);
                    Controls.Add(btn);
                }
            }
        }


        protected override void OnPreRender(EventArgs e)
        {
            if (!this.DesignMode && this.Enabled)
            {
                // Make sure ScriptManager exists
                _sm = ScriptManager.GetCurrent(Page);
                if (_sm == null)
                    throw new HttpException("A ScriptManager control must exist on the page.");

                // Register as client control.
                _sm.RegisterScriptControl(this);


                this.Style.Add("display", "none");


                String cbReference =
                    Page.ClientScript.GetCallbackEventReference(this,
                                                                "arg", "ReceiveServerData", "context");
                String callbackScript;
                callbackScript = "function CallServer(arg, context)" +
                                 "{ " + cbReference + ";}";
                Page.ClientScript.RegisterClientScriptBlock(this.GetType(),
                                                            "CallServer", callbackScript, true);


                base.OnPreRender(e);
            }
        }

        protected override void Render(HtmlTextWriter writer)
        {
            if (!this.DesignMode && this.Enabled)
            {
                _sm.RegisterScriptDescriptors(this);

                base.Render(writer);
            }
        }



        public void RaiseCallbackEvent(String eventArgument)
        {
            // All we're doing here is resetting the sliding expiration
            // of the forms authentication.  No additional code is needed.
        }

        public String GetCallbackResult()
        {
            // return an emtpy string.  We're not really interested
            // in the return value.
            return String.Empty;
        }

        #region IScriptControl Members

        IEnumerable<ScriptDescriptor> IScriptControl.GetScriptDescriptors()
        {
            if (this.Enabled)
            {
                ScriptControlDescriptor descriptor = new ScriptControlDescriptor(
                    "TSC.Timeout.Timeout", this.ClientID);

                descriptor.AddProperty("timeoutMinutes", this.TimeoutMinutes);
                descriptor.AddProperty("aboutToTimeoutMinutes", this.AboutToTimeoutMinutes);
                descriptor.AddProperty("timeoutURL", Page.ResolveClientUrl(this.TimeoutURL));
                descriptor.AddProperty("clientId", this.ClientID);
                descriptor.AddProperty("displayButton", this.DisplayButton);
                if (this.DisplayButton)
                    descriptor.AddProperty("btnClientId", this.btn.ClientID);
                return new ScriptDescriptor[] {descriptor};
            }
            else
                return null;
        }

        IEnumerable<ScriptReference> IScriptControl.GetScriptReferences()
        {
            if (this.Enabled)
            {
                ScriptReference reference = new ScriptReference();
                if (Page != null)
                    reference.Path = Page.ClientScript.GetWebResourceUrl(this.GetType(),
                                                                         "TSC.Timeout.Timeout.js");

                return new ScriptReference[] {reference};
            }
            else
                return null;
        }

        #endregion
    }
}